@@ -8,15 +8,18 @@ from scipy.sparse import issparse
 import numpy as np
 import pandas as pd
 import traceback
-from .constants import SKLearn
+from .constants import ModelTask, SKLearn
 from .error_handling import _format_exception
-from ._input_processing import _serialize_json_safe
+from responsibleai.serialization_utilities import serialize_json_safe
 from erroranalysis._internal.error_analyzer import (
     ModelAnalyzer, PredictionsAnalyzer)
+from erroranalysis._internal.metrics import metric_to_func
+from erroranalysis._internal.constants import Metrics, metric_to_display_name
 
 
 FEATURE_NAMES = ExplanationDashboardInterface.FEATURE_NAMES
 ENABLE_PREDICT = ErrorAnalysisDashboardInterface.ENABLE_PREDICT
+ROOT_COVERAGE = 100
 
 
 class ErrorAnalysisDashboardInput:
@@ -28,10 +31,13 @@ class ErrorAnalysisDashboardInput:
             true_y,
             classes,
             features,
-            locale,
             categorical_features,
             true_y_dataset,
-            pred_y):
+            pred_y,
+            model_task,
+            metric,
+            max_depth,
+            num_leaves):
         """Initialize the ErrorAnalysis Dashboard Input.
 
         :param explanation: An object that represents an explanation.
@@ -57,7 +63,7 @@ class ErrorAnalysisDashboardInput:
         :type classes: numpy.array or list[]
         :param features: Feature names.
         :type features: numpy.array or list[]
-            :param categorical_features: The categorical feature names.
+        :param categorical_features: The categorical feature names.
         :type categorical_features: list[str]
         :param true_y_dataset: The true labels for the provided dataset.
         Only needed if the explanation has a sample of instances from the
@@ -67,6 +73,26 @@ class ErrorAnalysisDashboardInput:
             alternative to the model and explanation for a more limited
             view.
         :type pred_y: numpy.ndarray or list[]
+        :param model_task: Optional parameter to specify whether the model
+            is a classification or regression model. In most cases, the
+            type of the model can be inferred based on the shape of the
+            output, where a classifier has a predict_proba method and
+            outputs a 2 dimensional array, while a regressor has a
+            predict method and outputs a 1 dimensional array.
+        :type model_task: str
+        :param metric: The metric name to evaluate at each tree node or
+            heatmap grid.  Currently supported classification metrics
+            include 'error_rate', 'recall_score', 'precision_score',
+            'f1_score', and 'accuracy_score'. Supported regression
+            metrics include 'mean_absolute_error', 'mean_squared_error',
+            'r2_score', and 'median_absolute_error'.
+        :type metric: str
+        :param max_depth: The maximum depth of the surrogate tree trained
+            on errors.
+        :type max_depth: int
+        :param num_leaves: The number of leaves of the surrogate tree
+            trained on errors.
+        :type num_leaves: int
         """
         self._model = model
         full_dataset = dataset
@@ -85,6 +111,8 @@ class ErrorAnalysisDashboardInput:
         self.dashboard_input = {}
         has_explanation = explanation is not None
         feature_length = None
+        self._max_depth = max_depth
+        self._num_leaves = num_leaves
 
         if has_explanation:
             if classes is None:
@@ -94,6 +122,12 @@ class ErrorAnalysisDashboardInput:
             dataset, true_y = self.input_explanation(explanation,
                                                      dataset,
                                                      true_y)
+            row_length = len(dataset)
+            # Only check dataset on explanation for row length bounds
+            if row_length > 100000:
+                raise ValueError(
+                    "Exceeds maximum number of rows"
+                    "for visualization (100000)")
 
         if classes is not None:
             classes = self._convert_to_list(classes)
@@ -104,6 +138,7 @@ class ErrorAnalysisDashboardInput:
 
         if isinstance(dataset, pd.DataFrame) and hasattr(dataset, 'columns'):
             self._dataframeColumns = dataset.columns
+            self._dfdtypes = dataset.dtypes
         try:
             list_dataset = self._convert_to_list(dataset)
         except Exception as ex:
@@ -141,10 +176,6 @@ class ErrorAnalysisDashboardInput:
         row_length = 0
         if list_dataset is not None:
             row_length, feature_length = np.shape(list_dataset)
-            if row_length > 100000:
-                raise ValueError(
-                    "Exceeds maximum number of rows"
-                    "for visualization (100000)")
             if feature_length > 1000:
                 raise ValueError("Exceeds maximum number of features for"
                                  " visualization (1000). Please regenerate the"
@@ -153,7 +184,7 @@ class ErrorAnalysisDashboardInput:
                                  " dataset.")
             self.dashboard_input[
                 ExplanationDashboardInterface.TRAINING_DATA
-            ] = _serialize_json_safe(list_dataset)
+            ] = serialize_json_safe(list_dataset)
             self.dashboard_input[
                 ExplanationDashboardInterface.IS_CLASSIFIER
             ] = self._is_classifier
@@ -194,20 +225,74 @@ class ErrorAnalysisDashboardInput:
             self.dashboard_input[
                 ExplanationDashboardInterface.PROBABILITY_Y
             ] = probability_y
-        if locale is not None:
-            self.dashboard_input[ExplanationDashboardInterface.LOCALE] = locale
         if model_available:
-            self._error_analyzer = ModelAnalyzer(model, full_dataset,
-                                                 full_true_y, features,
-                                                 categorical_features)
+            self._error_analyzer = ModelAnalyzer(model,
+                                                 full_dataset,
+                                                 full_true_y,
+                                                 features,
+                                                 categorical_features,
+                                                 model_task,
+                                                 metric)
         else:
-            self._error_analyzer = PredictionsAnalyzer(pred_y, full_dataset,
-                                                       full_true_y, features,
-                                                       categorical_features)
+            # Model task cannot be unknown when passing predictions
+            # Assume classification for backwards compatibility
+            if model_task == ModelTask.UNKNOWN:
+                model_task = ModelTask.CLASSIFICATION
+            self._error_analyzer = PredictionsAnalyzer(pred_y,
+                                                       full_dataset,
+                                                       full_true_y,
+                                                       features,
+                                                       categorical_features,
+                                                       model_task,
+                                                       metric)
         if self._categorical_features:
             self.dashboard_input[
                 ExplanationDashboardInterface.CATEGORICAL_MAP
             ] = self._error_analyzer.category_dictionary
+        # Compute metrics on all data cohort
+        if self._error_analyzer.model_task == ModelTask.CLASSIFICATION:
+            if self._error_analyzer.metric is None:
+                metric = Metrics.ERROR_RATE
+            else:
+                metric = self._error_analyzer.metric
+        else:
+            if self._error_analyzer.metric is None:
+                metric = Metrics.MEAN_SQUARED_ERROR
+            else:
+                metric = self._error_analyzer.metric
+        if model_available and true_y_dataset is not None:
+            full_predicted_y = self.compute_predicted_y(model, full_dataset)
+        else:
+            full_predicted_y = predicted_y
+        self.set_root_metric(full_predicted_y, full_true_y, metric)
+
+    def set_root_metric(self, predicted_y, true_y, metric):
+        if metric != Metrics.ERROR_RATE:
+            metric_func = metric_to_func[metric]
+            metric_value = metric_func(predicted_y, true_y)
+        else:
+            total = len(true_y)
+            if total == 0:
+                metric_value = 0
+            else:
+                if not isinstance(predicted_y, np.ndarray):
+                    predicted_y = np.array(predicted_y)
+                if not isinstance(true_y, np.ndarray):
+                    true_y = np.array(true_y)
+                diff = predicted_y != true_y
+                error = sum(diff)
+                metric_value = error / total
+        metric_name = metric_to_display_name[metric]
+        root_stats = {
+            ExplanationDashboardInterface.ROOT_METRIC_NAME: metric_name,
+            ExplanationDashboardInterface.ROOT_METRIC_VALUE: metric_value,
+            ExplanationDashboardInterface.ROOT_TOTAL_SIZE: len(true_y),
+            ExplanationDashboardInterface.ROOT_ERROR_COVERAGE: ROOT_COVERAGE
+        }
+
+        self.dashboard_input[
+            ExplanationDashboardInterface.ROOT_STATS
+        ] = root_stats
 
     def compute_predicted_y(self, model, dataset):
         predicted_y = None
@@ -333,7 +418,9 @@ class ErrorAnalysisDashboardInput:
     def debug_ml(self, features, filters, composite_filters):
         try:
             json_tree = self._error_analyzer.compute_error_tree(
-                features, filters, composite_filters)
+                features, filters, composite_filters,
+                max_depth=self._max_depth,
+                num_leaves=self._num_leaves)
             return {
                 WidgetRequestResponseConstants.DATA: json_tree
             }
@@ -383,6 +470,7 @@ class ErrorAnalysisDashboardInput:
         try:
             if self._dataframeColumns is not None:
                 data = pd.DataFrame(data, columns=self._dataframeColumns)
+                data = data.astype(dict(self._dfdtypes))
             if (self._is_classifier):
                 model_pred_proba = self._model.predict_proba(data)
                 prediction = self._convert_to_list(model_pred_proba)
