@@ -8,7 +8,7 @@ import numpy as np
 import pandas as pd
 from .constants import SKLearn
 from .error_handling import _format_exception
-from ._input_processing import _serialize_json_safe
+from responsibleai.serialization_utilities import serialize_json_safe
 
 
 class ExplanationDashboardInput:
@@ -19,8 +19,7 @@ class ExplanationDashboardInput:
             dataset,
             true_y,
             classes,
-            features,
-            locale):
+            features):
         """Initialize the Explanation Dashboard Input.
 
         :param explanation: An object that represents an explanation.
@@ -87,6 +86,7 @@ class ExplanationDashboardInput:
 
         if isinstance(dataset, pd.DataFrame) and hasattr(dataset, 'columns'):
             self._dataframeColumns = dataset.columns
+            self._dfdtypes = dataset.dtypes
         try:
             list_dataset = self._convert_to_list(dataset)
         except Exception as ex:
@@ -128,7 +128,7 @@ class ExplanationDashboardInput:
                                  " dataset.")
             self.dashboard_input[
                 ExplanationDashboardInterface.TRAINING_DATA
-            ] = _serialize_json_safe(list_dataset)
+            ] = serialize_json_safe(list_dataset)
             self.dashboard_input[
                 ExplanationDashboardInterface.IS_CLASSIFIER
             ] = self._is_classifier
@@ -250,13 +250,12 @@ class ExplanationDashboardInput:
             self.dashboard_input[
                 ExplanationDashboardInterface.PROBABILITY_Y
             ] = probability_y
-        if locale is not None:
-            self.dashboard_input[ExplanationDashboardInterface.LOCALE] = locale
 
     def on_predict(self, data):
         try:
             if self._dataframeColumns is not None:
                 data = pd.DataFrame(data, columns=self._dataframeColumns)
+                data = data.astype(dict(self._dfdtypes))
             if (self._is_classifier):
                 prediction = self._convert_to_list(
                     self._model.predict_proba(data))
